
# q-expansion of newform 1716.2.a.f, downloaded from the LMFDB on 06 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1716
weight = 2
poly_data = [-14, -12, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[0, 1, 0], 1], [[-8, -2, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0], [-1, 0, 0], [0, 1, 0], [0, 0, 1], [-1, 0, 0], [-1, 0, 0], [2, 1, 1], [0, 0, -1], [0, 0, 1], [0, 1, -1], [2, -1, 0], [4, 0, 0], [2, 0, -1], [0, -1, 1], [2, 2, 2], [2, -2, -2], [4, 2, 0], [4, -2, -2], [2, -1, 2], [-2, 0, -4], [8, -2, 1], [8, 1, 1], [-4, -2, 2], [2, -3, 0], [8, 0, 0], [-2, -1, 1], [8, 0, 0], [-8, 2, 0], [12, 2, 3], [6, 2, 0], [2, -3, -1], [4, 4, 6], [2, 1, 0], [6, -3, -3], [-2, -4, 3], [0, 0, -5], [6, 2, -5], [0, 5, -2], [-8, -4, 2], [-6, -1, 3], [-4, 2, 3], [-2, 2, 5], [4, 2, -2], [-4, -2, -1], [6, -2, -3], [0, 4, 0], [-8, 1, 3], [6, -3, -4], [-10, 4, -1], [8, -6, -2], [-12, -1, -5], [-10, -4, -3], [2, 2, -4], [0, -6, -6], [6, -4, 2], [-16, 2, 0], [6, 2, 2], [-4, -6, -7], [-2, 4, 2], [2, 2, 3], [-14, -1, 1], [10, -6, 1], [4, 0, 5], [-4, 0, 3], [-6, -4, -1], [-6, -7, 4], [-12, 5, 0], [6, -6, 4], [-10, -4, 0], [-6, 6, 4], [-4, -1, 0], [-2, 0, 3], [-4, -2, 2], [2, -6, 0], [16, 3, 6], [0, -2, 6], [-2, -6, -4], [-20, -4, -8], [-4, 3, -2], [-8, -2, -3], [-8, 2, 3], [2, 0, -6], [0, -4, -2], [6, 8, -4], [-4, -1, -3], [-20, -4, -4], [32, 1, 2], [10, 4, 2], [-2, -8, -5], [4, 5, -8], [-4, 4, -3], [-6, 8, -1], [16, 3, 6], [-2, 4, -6], [18, -1, 0], [20, 6, 4], [14, 7, 8], [14, -4, -4], [-4, -3, -1], [-6, 0, 0], [2, -1, -1], [6, 0, 3], [-8, -8, -6], [-14, -5, 7], [-20, 1, 1], [-14, -8, -14], [4, -10, -8], [26, 2, -3], [12, 2, 2], [-6, 0, -4], [10, -3, -5], [-4, 2, 1], [-20, -3, 2], [20, -9, 0], [12, 1, 4], [-2, 4, -6], [6, -11, -6], [-4, 2, -6], [-6, 0, 2], [14, -2, -6], [-14, -2, 10], [-8, 12, 0], [-24, -5, 1], [2, -12, 0], [16, 5, 4], [-2, 11, 9], [8, -6, -4], [24, -8, -4], [-8, -6, 6], [12, -6, -7], [-8, -8, -5], [20, -2, 8], [-4, -6, -2], [-14, -2, 3], [2, -6, 3], [22, 0, -4], [-6, -7, -4], [-8, 8, 7], [6, -12, -4], [-2, -1, -5], [-12, 4, 1], [10, 6, -1], [8, -2, -6], [-22, 0, 3], [18, 4, 0], [4, -12, -2], [10, 0, -6], [-12, 5, 3], [-8, -2, -14], [28, 2, -2], [-18, -2, 0], [22, -4, -8], [20, -4, -8], [-2, 6, -10], [-32, 2, 2], [32, 2, -2], [34, 1, -5], [30, 1, -2], [4, -4, -2], [10, 4, -9], [-18, -2, -8], [20, 1, 5], [8, 10, 3], [24, -4, 7], [6, 11, -4], [-12, 8, -2], [-12, 14, 4], [2, 2, 12], [16, 2, 9], [6, -12, 2], [-8, -2, -12], [8, -4, -4], [-10, -10, -12], [-10, -2, -4], [12, 8, 0], [26, 4, -8], [-4, 16, 1], [2, 1, 14], [-12, 9, 7], [-6, -2, 12], [10, 3, -12], [-28, -4, 0], [14, -14, -5], [-2, -4, -1], [-10, -2, 0], [26, 2, 10], [-28, 8, 0], [24, 3, 10], [14, 2, 0], [8, -10, -6], [-34, -6, -9], [-8, 0, 2], [16, 0, 12], [6, 7, 10], [-28, -4, 6], [14, 4, -4], [-14, -8, 4], [-26, 6, 0], [22, 6, 17], [6, 6, -6], [-14, -10, -9], [-18, 1, 7], [34, 8, -4], [8, -6, 0], [14, 2, -12], [-6, 10, 4], [44, 5, 2], [36, 8, -2], [-26, -6, -1], [-24, 4, -8], [-12, -12, 6], [-18, 0, 0], [-4, -6, -18], [-12, -4, -10], [10, 4, 7], [-20, -2, 14], [8, 7, 13], [-8, -7, -19], [-14, 8, 8], [0, -3, -10], [-14, -12, 10], [-4, 14, 1], [12, -9, 6], [22, -3, 12], [-30, 2, -4], [10, 14, -2], [-10, 14, 0], [44, 2, -6], [4, -15, -5], [-32, -2, 6], [-34, -4, -4], [12, 11, -13], [4, 4, -1], [-16, -1, -11], [20, 0, 0], [-8, 4, 12], [-10, 14, 0], [-22, 14, -5], [0, 14, 3], [0, 4, 21], [0, -6, 6], [-30, -5, -3], [24, 12, -10], [14, -14, -4], [-22, -6, 9], [-2, 18, -6], [-4, 11, -10], [-10, -2, 8], [16, -12, -9], [18, -2, 6], [10, 6, -10], [6, 8, -9], [10, -4, 8], [-6, 0, 7], [-36, -1, -9], [-26, 16, 5], [-18, 4, 9], [-20, 8, 5], [10, -14, -6], [-6, 6, -14], [18, -5, 3], [-22, -10, -8], [18, 10, 18], [-14, 14, -4], [-14, 5, 10], [-12, -4, -16], [20, -9, -8], [-4, 13, 14], [-8, 12, 9], [-48, -7, 7], [6, 10, 12], [-2, 5, 12], [-14, -6, 0], [-18, -11, -9], [-6, -6, 10], [28, -11, 4], [10, 0, -4], [12, 14, 17], [-16, 14, 3], [-10, 14, 8], [50, 8, 8], [0, -5, -6], [-66, -2, -4], [64, -2, -5], [20, -2, 13], [56, -4, 0], [-6, 24, 12], [-2, -6, -17], [20, -19, 4], [16, 15, 11], [-6, 4, 14], [-24, -4, -9], [30, 4, -7], [-14, 12, 1], [24, -10, -12], [-70, 3, 3], [40, -2, 6], [18, -14, 6], [10, -20, 6], [-8, -8, 11], [-14, -8, -6], [26, 0, 1], [-46, -9, -3], [24, -4, -15], [-44, 0, 6], [36, 8, -10], [-28, -6, -19], [34, 10, -4], [-28, 0, 6], [14, -6, 6], [36, -14, -6], [20, 13, 6], [-30, -3, 15], [-20, 10, -12], [12, 14, -14], [10, 6, 12], [-16, -12, -18], [22, -12, -13], [-2, -10, 14], [-4, -21, -11], [22, -5, -7], [-18, 12, 8], [24, -15, -3], [20, -15, -4], [6, -16, -9], [10, 18, 3], [18, 7, -4], [56, 7, 12], [-12, -14, 4], [-32, 5, -7], [8, 10, -11], [-34, 16, 2], [22, 11, -13], [-10, 20, -9], [-16, 10, -5], [20, -2, 4], [4, -8, -4], [38, -2, 1], [18, 10, 14], [10, 11, -1], [-8, 24, 0], [-30, -14, 12], [46, 12, 3], [-4, -14, 4], [-10, 23, 8], [4, -18, 10], [42, -8, -8], [-36, -11, -12], [-14, 0, -1], [-2, 0, -6], [24, 0, -3], [6, -22, -2], [-12, -11, 16], [24, 15, 19], [20, -18, -8], [-38, -9, -5], [42, -10, 4], [18, 20, 17], [6, -5, 19], [-6, -6, 6], [38, -16, -2], [44, -11, -1], [-12, -6, -18], [-38, 20, 9], [8, -20, 0], [14, -6, 17], [22, -6, -17], [-24, 18, 9], [-4, -16, -14], [40, -10, -2], [24, 12, -4], [-62, -4, -1], [-2, 13, -3], [-12, 11, -11], [-62, 8, -4], [42, 2, -6], [8, -14, -12], [44, -10, -5], [-28, -17, 1], [0, 14, -10], [-8, 3, 20], [32, 11, 18], [28, 16, 9], [-46, 12, 1], [-60, 10, 1], [62, 4, -6], [6, -11, -2], [14, -18, 12], [0, -8, -21], [8, 2, 13], [20, 12, -4], [18, 5, -4], [-18, -16, 14], [0, 4, 24], [44, 2, -12], [-20, 14, 2], [8, 0, -19], [10, -16, -8], [56, 4, 0], [18, 12, -5], [58, 4, 2], [-10, -20, -22], [44, 8, 11], [-6, -3, 14], [-38, 20, 9], [14, -9, 11], [12, -14, 4], [-2, -4, 3], [14, -3, -10], [-22, 16, -10], [-10, -4, -28], [2, 15, 17], [-32, -2, 18], [-12, -5, -5], [50, 2, -6], [6, 16, -2], [-54, -8, -7], [-12, -14, -14], [32, -19, -4], [42, 4, -12], [-50, -8, -4], [58, -10, 4], [-30, -4, 20], [20, 2, -12], [-14, -8, -18], [4, 13, 24], [16, 12, 8], [-14, -12, -29], [-48, -7, -12], [-14, 16, 0]]
